#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const rollup = require("rollup");
const typescript = require("rollup-plugin-typescript2");
const generatePackageJson = require("rollup-plugin-generate-package-json");
const autoExternal = require("rollup-plugin-auto-external");
const globby = require("globby");
const mv = require("mv");
async function build(pkg) {
    const options = pkg.buildOptions;
    const inputOptions = {
        input: options.input,
        plugins: [
            typescript({
                objectHashIgnoreUnknownHack: true
            }),
            autoExternal(),
            generatePackageJson({
                baseContents: rewritePackageJson({
                    pkg,
                    preserved: []
                }),
                additionalDependencies: Object.keys(pkg.dependencies)
            })
        ],
        inlineDynamicImports: true
    };
    // create a bundle
    const bundle = await rollup.rollup(inputOptions);
    // generates
    const commonOutputOptions = {
        preferConst: true,
        sourcemap: options.sourcemap
    };
    const generates = [
        {
            ...commonOutputOptions,
            file: pkg.main,
            format: "cjs"
        },
        {
            ...commonOutputOptions,
            file: pkg.module,
            format: "esm"
        }
    ];
    if (options.umd) {
        generates.push({
            ...commonOutputOptions,
            file: pkg.umd,
            format: "umd",
            name: options.umd
        });
    }
    await Promise.all(generates.map(outputOptions => bundle.write(outputOptions)));
    await rewriteDeclarations(pkg);
    if (pkg.buildOptions.bin) {
        await Promise.all(Object.keys(pkg.buildOptions.bin).map(async (alias) => {
            const options = pkg.buildOptions.bin[alias];
            const inputOptions = {
                input: options.input,
                plugins: [
                    typescript({
                        objectHashIgnoreUnknownHack: true
                    }),
                    autoExternal()
                ],
                inlineDynamicImports: true
            };
            const bundle = await rollup.rollup(inputOptions);
            await bundle.write({
                banner: `#!/usr/bin/env node`,
                preferConst: true,
                sourcemap: options.sourcemap,
                file: pkg.bin[alias],
                format: "cjs"
            });
        }));
    }
}
async function main() {
    const cwd = process.cwd();
    const pkg = readPackageJson(cwd);
    await build(pkg);
}
main().catch(err => {
    console.error(err);
    process.exit(1);
});
//
function readPackageJson(baseDir) {
    return JSON.parse(fs_1.readFileSync(path_1.resolve(baseDir, "package.json"), {
        encoding: "utf-8"
    }));
}
function rewritePackageJson({ pkg, preserved }) {
    const newPkg = {};
    const fields = [
        "name",
        "version",
        "description",
        "sideEffects",
        "peerDependencies",
        "repository",
        "homepage",
        "keywords",
        "author",
        "license",
        "engines"
    ];
    if (preserved) {
        fields.push(...preserved);
    }
    fields.forEach(field => {
        if (pkg[field]) {
            newPkg[field] = pkg[field];
        }
    });
    function transformPath(filepath) {
        return path_1.basename(filepath);
    }
    newPkg.main = transformPath(pkg.main);
    newPkg.module = transformPath(pkg.module);
    newPkg.typings = transformPath(pkg.typings);
    newPkg.typescript = pkg.typescript;
    if (newPkg.typescript.definition) {
        newPkg.typescript.definition = transformPath(pkg.typescript.definition);
    }
    if (pkg.bin) {
        newPkg.bin = {};
        for (const alias in pkg.bin) {
            newPkg.bin[alias] = transformPath(pkg.bin[alias]);
        }
    }
    // if (produceUMD) {
    //   newPkg.unpkg = pkg.typings.replace('dist/', '');
    //   newPkg.umd = pkg.typings.replace('dist/', '');
    // }
    return newPkg;
}
function pathOf(filepath) {
    const dirs = filepath.split("/");
    return dirs.slice(0, dirs.length - 1).join("/");
}
function depthOf(filepath) {
    return filepath.split("/").length;
}
async function rewriteDeclarations(pkg) {
    if (!pkg.typings) {
        return;
    }
    const declarationFiles = await globby(["**/*.d.ts", "**/*.d.ts.map"], {
        absolute: false,
        cwd: path_1.resolve(process.cwd(), pathOf(pkg.typings))
    });
    // find possible `pkg.typings` files
    const possibleIndexes = declarationFiles.filter(filepath => filepath.endsWith(path_1.basename(pkg.typings)));
    const indexFile = possibleIndexes.reduce((a, b) => {
        const aDirs = depthOf(a);
        const bDirs = depthOf(b);
        return aDirs < bDirs ? a : b;
    });
    const lvl = depthOf(indexFile);
    // no need to move it to /dist
    if (lvl <= 1) {
        return;
    }
    const distPath = pathOf(pkg.typings);
    const baseToRemove = pathOf(indexFile)
        .split("/")
        .filter((_, i) => i < lvl)
        .join("/") + "/";
    await Promise.all(declarationFiles.map(async (filepath) => {
        await move(path_1.resolve(process.cwd(), distPath, filepath), path_1.resolve(distPath, filepath.replace(baseToRemove, "")));
    }));
}
function move(src, dest) {
    return new Promise((resolve, reject) => {
        mv(src, dest, { mkdirp: true }, err => {
            if (err) {
                reject(err);
            }
            else {
                resolve();
            }
        });
    });
}
